#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>

/* simple print error and exit routine */
#define error(string)                                                   \
  do {                                                                  \
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );        \
      exit( EXIT_FAILURE );                                             \
     } while (0)

/*
  invoke as

  create

*/



int 
main (int argc, char *argv[])
{
  BPipe *bpipe;
  BPipeOutput *bpo;

  void *data;
  double *weight;

  size_t i;
  size_t nphots = 1000;
  
  if ( argc > 1 )
    nphots = atoi( argv[1] );


  if ( NULL == ( bpipe = bpipe_new( ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* create new data packet fields */

  if ( bpipe_dpktf_add( bpipe,  "weight",     BPDType_double,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, "stdout" ) ) )
    error( bpipe_strerror( bpipe_errno ) );
    
  /* map data packet fields */
  if ( NULL == ( data = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* output header */
  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* get handles to data packet fields */
  weight    = bpipe_dpktf_datap( bpipe, data, "weight",    BPDType_double   );

  *weight = 0.75;

  for (i = 1; i <= nphots ; i++)
    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
      error( "error writing photon" );

  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}

